/*
    gUserMan- A GTK+ User manager
    Copyright (C) 1999 Scott Fritzinger 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "main.h"

extern gint	config_copy_skel;
extern gint	config_create_dir;
extern gint     config_min;
extern gint     config_max;
extern gint     config_gid;
extern gchar    config_dir[1024];
extern gchar    config_shell[1024];
extern gint     config_warn;
extern gint     config_inactive;
extern gint     config_expire;
extern gint     config_never;


void message_window (gchar *title, gchar *message ) {

	/* Displays a dialog with window name "title" and shows "message" */

        GtkWidget *dialog, *label, *button;

        dialog = gtk_dialog_new();
	GTK_WINDOW(dialog)->type = GTK_WINDOW_DIALOG;
        gtk_window_set_title(GTK_WINDOW(dialog), title);
	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);
        gtk_container_border_width(GTK_CONTAINER(dialog), 5);
        label = gtk_label_new(message);
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
        button = gtk_button_new_with_label("OK");
        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                            GTK_SIGNAL_FUNC(gtk_widget_destroy),
                            GTK_OBJECT(dialog));
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                           label, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                           button, TRUE, TRUE, 0);
        gtk_widget_show(label);
        gtk_widget_show(button);   
        gtk_widget_show(dialog);
        gtk_widget_grab_default (button);
}

void message_window_long (gchar *title, gchar *message ) {

	/* Displays a dialog with window name "title" and shows "message" */

        GtkWidget *dialog, *scrwin, *label, *button, *cbutton;

        dialog = gtk_dialog_new();
	GTK_WINDOW(dialog)->type = GTK_WINDOW_DIALOG;
        gtk_window_set_title(GTK_WINDOW(dialog), title);
	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);
        gtk_container_border_width(GTK_CONTAINER(dialog), 5);

        label = gtk_label_new(message);
        gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), 0,0);
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);

	scrwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrwin);
	gtk_widget_set_usize(scrwin, 400, 150);
        gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		scrwin);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrwin),
		label);

        button = gtk_button_new_with_label("OK");
        gtk_widget_show(button);   
        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                            GTK_SIGNAL_FUNC(gtk_widget_destroy),
                            GTK_OBJECT(dialog));
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                           button, TRUE, TRUE, 0);

        gtk_widget_show(dialog);
        gtk_widget_grab_default (button);

	cbutton = gtk_button_new();
	gtk_widget_show(cbutton);

	(void)wait_for_hide(dialog, button, cbutton);

	if (GTK_IS_OBJECT(dialog))
		gtk_widget_destroy(dialog);
}

gint confirm_window (gchar *title, gchar *message) {

	/* Displays the message, along with a Yes and No button.
	 * returns 1 if yes is clicked
	 * returns 0 if no  is clicked
	 */

	GtkWidget *dialog, *label, *yes, *no;
	gint retval=0;

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), title);
	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);
	GTK_WINDOW(dialog)->type = GTK_WINDOW_DIALOG;

	label = gtk_label_new(message);
	gtk_widget_show(label);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		label);

	yes = gtk_button_new_with_label("Yes");
	gtk_widget_show(yes);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		yes);

	no = gtk_button_new_with_label("No");
	gtk_widget_show(no);
	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		no);

	gtk_widget_show(dialog);

	retval = wait_for_hide(dialog, yes, no);
	if (GTK_IS_OBJECT(dialog))
		gtk_widget_destroy(dialog);
	return (retval);
}

void wait_for_hide_ok_click (GtkWidget *dialog) {

        gtk_object_set_data(GTK_OBJECT(dialog), "button", "OK");
        gtk_widget_hide(dialog);
}

gint wait_for_hide (GtkWidget *dialog, GtkWidget *ok_button,
                   GtkWidget *cancel_button) {
                   
	gint cont=1;
	
        gtk_object_set_data(GTK_OBJECT(dialog), "button", "CANCEL");
        gtk_signal_connect_object(
                        GTK_OBJECT(ok_button), "clicked",
                        GTK_SIGNAL_FUNC(wait_for_hide_ok_click),
                        GTK_OBJECT(dialog));
        gtk_signal_connect_object(GTK_OBJECT(cancel_button),
                        "clicked",
                        GTK_SIGNAL_FUNC(gtk_widget_hide),
                        GTK_OBJECT(dialog));
        gtk_widget_show(dialog);
        while (cont) {
		/* because the window manager could destroy the window */
		if(!GTK_IS_OBJECT(dialog))
			return 0;
		if (GTK_WIDGET_VISIBLE(dialog))
			gtk_main_iteration();
		else
			cont = 0;
	}
        if (strcmp("CANCEL",
           (gchar*)gtk_object_get_data(GTK_OBJECT(dialog), "button"))==0)
                return 0;
        return 1;
}

int num_days_in_month (int month) {

	switch (month) {
		case 1:
			return (31);
			break;
		case 2:
			return (28);
			break;
		case 3:
			return (31);
			break;
		case 4:
			return (30);
			break;
		case 5:
			return (31);
			break;
		case 6:
			return (30);
			break;
		case 7:
			return (31);
			break;
		case 8:
			return (31);
			break;
		case 9:
			return (30);
			break;
		case 10:
			return (31);
			break;
		case 11:
			return (30);
			break;
		case 12:
			return (31);
			break;
		default:
			return (0);
	}
}

time_t date_to_time_t (int month, int day, int year) {

	int diff_year      = 0;
	int sum_days       = 0;
	int num_leap_years = 0;

	time_t t;

	diff_year = year - 1970;

	if (year % 4 == 0)
		num_leap_years = (diff_year + 2)/4 - 1;
	   else
		num_leap_years = (diff_year + 2)/4;

	if (month > 1)
		sum_days += num_days_in_month(1);
	if (month > 2) {
		if (year % 4 == 0)
			sum_days += num_days_in_month(2) + 1;
		   else
			sum_days += num_days_in_month(2);
	}
	if (month > 3)
		sum_days += num_days_in_month(3);
	if (month > 4)
		sum_days += num_days_in_month(4);
	if (month > 5)
		sum_days += num_days_in_month(5);
	if (month > 6)
		sum_days += num_days_in_month(6);
	if (month > 7)
		sum_days += num_days_in_month(7);
	if (month > 8)
		sum_days += num_days_in_month(8);
	if (month > 9)
		sum_days += num_days_in_month(9);
	if (month > 10)
		sum_days += num_days_in_month(10);
	if (month > 11)
		sum_days += num_days_in_month(11);

	if ((year % 4 == 0) && (month == 2))
		t = diff_year*31536000+(num_leap_years + sum_days + day)*86400
		+ num_days_in_month(2)+1 * 86400;
	   else
		t = diff_year*31536000+(num_leap_years + sum_days + day)*86400
		+ num_days_in_month(month) * 86400;

	if (TRUE) {
	struct tm *tm;
	tm = localtime (&t);
/* Debug output info
	printf("num_leap_years: %i\n", num_leap_years);
	printf("            In: %i %i %i\n", month, day, year);
	printf("  Out (time_t): %li\n", t);
	printf("  Out   (date): %i %i %i\n",tm->tm_mon,tm->tm_mday,tm->tm_year);
*/
	}
	return (t);
}

void save_config () {

	gchar conf[1024];
	FILE *f = NULL;

	sprintf(conf, "%s/.guserman", getenv("HOME"));
	f = fopen(conf, "w");
	fprintf(f, "min=%i\n", config_min);
	fprintf(f, "max=%i\n", config_max);
	fprintf(f, "gid=%i\n", config_gid);
	fprintf(f, "dir=%s\n", config_dir);
	fprintf(f, "shell=%s\n", config_shell);
	fprintf(f, "warn=%i\n", config_warn);
	fprintf(f, "inactive=%i\n", config_inactive);
	fprintf(f, "expire=%i\n", config_expire);
	fprintf(f, "never=%i\n", config_never);
	fprintf(f, "copy_skel=%i\n", config_copy_skel);
	fprintf(f, "create_dir=%i\n", config_create_dir);
	fclose(f);
}


void read_config () {

	gchar conf[1024];
	FILE *f = NULL;

	sprintf(conf, "%s/.guserman", getenv("HOME"));

	f = fopen(conf, "r");
	if (f == NULL) {
		save_config(); /* save the defaults */
		f = fopen(conf, "r");
	}

	fscanf(f, "min=%i\n", &config_min);
	fscanf(f, "max=%i\n", &config_max);
	fscanf(f, "gid=%i\n", &config_gid);
	fscanf(f, "dir=%s\n", config_dir);
	fscanf(f, "shell=%s\n", config_shell);
	fscanf(f, "warn=%i\n", &config_warn);
	fscanf(f, "inactive=%i\n", &config_inactive);
	fscanf(f, "expire=%i\n", &config_expire);
	fscanf(f, "never=%i\n", &config_never);
	fscanf(f, "copy_skel=%i\n", &config_copy_skel);
	fscanf(f, "create_dir=%i\n", &config_create_dir);

	fclose (f);	

}
