/*
    gUserMan- A GTK+ User manager
    Copyright (C) 1999 Scott Fritzinger

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "main.h"

/* Some globals */
gint     users_changed    = 0;
gint	 groups_changed   = 0;
gint	 debug_mode       = 0;
gchar   *salt; 		  /* init'ed in main() */

/* The clist's for the groups and users */
GtkWidget *users_clist;
GtkWidget *groups_clist;


/* New user template global config variables */
gint	 config_copy_skel = 1;
gint	 config_create_dir= 1;
gint	 config_min	  = -1;
gint	 config_max	  = 99999;
gint	 config_gid	  = 100;
gchar	 config_dir[1024] = "/home";
gchar	 config_shell[1024] = "/bin/bash";
gint	 config_warn      = -1;
gint	 config_inactive  = -1;
gint	 config_expire    = 12; /* months after current day to expire */
gint	 config_never	  = 1;

int main (int argc, char **argv) {

	GtkWidget *window, *userman_widget;
        GtkWidget *dialog, *label, *button;
	gchar error_message[1024];
	gint show_main = 1;

	FILE *fp = NULL;
	
	gtk_init(&argc, &argv);

	/* This needs to be changed somehow. what is the 
	   "correct" way to create a salt string? */
	salt      = "Az3R";

	if (debug_mode)
		printf("Debug Mode\n");

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "gUserMan (gUM)");
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
	gtk_signal_connect_object(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(main_quit), NULL);
	gtk_widget_realize(window);

	strcpy(error_message, "");
	if ((fp = fopen(passwd_file, "r+")) == NULL) {
		strcat(error_message, "Can't open the passwd file\n");
		show_main = 0;}
	   else
		fclose(fp);

	if ((fp = fopen(shadow_file, "r+")) == NULL) {
		strcat(error_message, "Can't open the shadow file\n");
		show_main = 0;}
	   else
		fclose(fp);

	if ((fp = fopen(group_file, "r+")) == NULL) {
		strcat(error_message, "Can't open the group file\n");		
		show_main = 0;}
	   else
		fclose(fp);

	if (show_main) {
		userman_widget = create_userman(); /* Get the main UI */
		gtk_widget_show(userman_widget);
		gtk_container_add(GTK_CONTAINER(window), userman_widget);
		gtk_widget_show(window);}
	   else {
		strcat(error_message, 
"\nEither you don't have write permissions to the \
passwd/shadow/group files, or they are not where \
you specified at compile time. \
Edit the main.h file that came with the gUserMan \
software to specify where your passwd/shadow/group \
files are located. By default, gUserMan looks at \
/etc/passwd, /etc/shadow, and /etc/group");
        
	        dialog = gtk_dialog_new();
	        GTK_WINDOW(dialog)->type = GTK_WINDOW_DIALOG;
	        gtk_window_set_title(GTK_WINDOW(dialog), "Permissions Error");
	        gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
	        gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);   
	        gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	        label = gtk_label_new(error_message);
	        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	        button = gtk_button_new_with_label("OK");
	        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	        gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
	                            GTK_SIGNAL_FUNC(gtk_main_quit),NULL);
	        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
	                           label, FALSE, FALSE, 0);
	        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
	                           button, TRUE, TRUE, 0);
	        gtk_widget_show(label);
	        gtk_widget_show(button);
	        gtk_widget_show(dialog);
	        gtk_widget_grab_default (button);
	}

	gtk_main();

	return(0);
}

GtkWidget *create_userman () {

	GtkWidget *notebook, *users, *groups, *label;

	notebook = gtk_notebook_new();
	gtk_widget_show(notebook);
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), FALSE);

	/* Read in our configuration */
	read_config();

	/* the groups widget MUST be created FIRST! */
	groups = create_groups_widget();
	gtk_widget_show(groups);
	users = create_users_widget();
	gtk_widget_show(users);

	label = gtk_label_new("Users");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), users, label);

	label = gtk_label_new("Groups");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), groups, label);

	return (notebook);
}

gint main_quit (gpointer data) {

	if ((users_changed) && 
	    (confirm_window("Changes not saved",
	    "User changes have not been saved. Save now?"))) {
		save_users(users_clist);
	}

	if ((groups_changed) && 
	    (confirm_window("Changes not saved",
	    "Group changes have not been saved. Save now?"))) {
		save_groups(groups_clist);
	}

	gtk_main_quit();
	return (FALSE);
}
