/*
    gUserMan- A GTK+ User manager
    Copyright (C) 1999 Scott Fritzinger 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "main.h"

extern GtkWidget *groups_clist;

void free_passwd (struct passwd *pwent) {

	free(pwent->pw_name);
	free(pwent->pw_passwd);
	free(pwent->pw_gecos);
	free(pwent->pw_dir);
	free(pwent->pw_shell);
	free(pwent);
}

void free_spwd (struct spwd *spwent) {

	free(spwent->sp_namp);
	free(spwent->sp_pwdp);

	free(spwent);
}

void free_User (struct User *user) {

	free(user->login);
	free(user->name);
	free(user->dir);
	free(user->shell);
	free(user->password);

	free(user);
}

void free_group (struct group *grp) {

	int x=0;

	free(grp->gr_name);
	free(grp->gr_passwd);
	while (grp->gr_mem[x])
		x++;
	x--;
	while (x>=0) {
		free(grp->gr_mem[x]);
		x--;
	}
	free(grp->gr_mem);
	free(grp);
}

struct passwd *create_passwd (char *login, char *passwd,
			      int uid, int gid, char *name,
			      char *home, char *shell) {

	struct passwd *copy;

	copy = (struct passwd *)malloc(sizeof(struct passwd));
	copy->pw_name   = strdup(login);
	copy->pw_passwd = strdup(passwd);
	copy->pw_uid    = uid;
	copy->pw_gid    = gid;
	if (name)
		copy->pw_gecos  = strdup(name);
	   else
		copy->pw_gecos  = strdup("\0");
	
	if (home)
		copy->pw_dir    = strdup(home);
	   else
		copy->pw_dir    = strdup("\0");

	if (shell)
		copy->pw_shell  = strdup(shell);
	   else
		copy->pw_shell  = strdup("\0");
	return (copy);
}

struct spwd *create_spwd (char *login, char *passwd, 
			long int date_last_change,
			long int min_days_change,
			long int max_days_change,
			long int num_days_warning,
			long int num_days_inactive,
			long int day_expires,
			unsigned long int flag) {

	struct spwd *spwent;

	spwent = (struct spwd *)malloc(sizeof(struct spwd));

	spwent->sp_namp = strdup(login);
	spwent->sp_pwdp = strdup(passwd);
	spwent->sp_lstchg = date_last_change;
	spwent->sp_min = min_days_change;
	spwent->sp_max = max_days_change;
	spwent->sp_warn = num_days_warning;
	spwent->sp_inact = num_days_inactive;
	spwent->sp_expire = day_expires;
	spwent->sp_flag = flag;

	return (spwent);
}

struct group *create_group (char *name, char *password, int gid,
			    GSList *members) {

	struct group *grp;
	int x;

	grp = (struct group*)malloc(sizeof(struct group));
	grp->gr_name = strdup(name);
	grp->gr_passwd = strdup(password);
	grp->gr_gid = gid; 
	grp->gr_mem = (char **)malloc(sizeof(char*)*
		(g_slist_length(members)+1));
	grp->gr_mem[g_slist_length(members)] = NULL;
	for (x=0; x<g_slist_length(members); x++) {
		grp->gr_mem[x] = 
			strdup((g_slist_nth(members, x))->data);
	}
	return (grp);
}

struct User *create_User (struct passwd *pwent, struct spwd *spwent) {

	struct User *user;

	user = (struct User *)malloc(sizeof(struct User));
	user->login = strdup(pwent->pw_name);
	user->uid = pwent->pw_uid;
	user->gid = pwent->pw_gid;
	user->name = strdup(pwent->pw_gecos);
	user->dir = strdup(pwent->pw_dir);
	user->shell = strdup(pwent->pw_shell);
	user->password = strdup(spwent->sp_pwdp);
	user->date_last_change = spwent->sp_lstchg;
	user->min_days_change = spwent->sp_min;
	user->max_days_change = spwent->sp_max;
	user->num_days_warning = spwent->sp_warn;
	user->num_days_inactive = spwent->sp_inact;
	user->day_expires = spwent->sp_expire;
	user->flag = spwent->sp_flag;

	return (user);
}

struct passwd *copy_passwd (struct passwd *pwent) {

	return (create_passwd(pwent->pw_name,pwent->pw_passwd,
		pwent->pw_uid,pwent->pw_gid,pwent->pw_gecos,
		pwent->pw_dir,pwent->pw_shell));
}

struct spwd *copy_spwd (struct spwd *spwent) {

	return (create_spwd(spwent->sp_namp, spwent->sp_pwdp,
		spwent->sp_lstchg, spwent->sp_min, spwent->sp_max,
		spwent->sp_warn, spwent->sp_inact, spwent->sp_expire,
		spwent->sp_flag));
}

struct group *copy_group (struct group *grp) {

	GSList *members = NULL, *node;
	struct group *grp2;
	int x = 0;

	while (grp->gr_mem[x]) {
		members = g_slist_append(members, strdup(grp->gr_mem[x]));
		x++;
	}

	grp2 = create_group(grp->gr_name, grp->gr_passwd,
		grp->gr_gid, members);

	for (node=members; node; node = node->next)
		free(node->data);
	g_slist_free(members);

	return (grp2);
}

struct passwd *create_passwd_from_User (struct User *user) {

	return (create_passwd(user->login, "x", user->uid, user->gid,
		user->name, user->dir, user->shell));
}

struct spwd *create_spwd_from_User (struct User *user) {

	return (create_spwd(user->login, user->password,
		user->date_last_change, user->min_days_change,
		user->max_days_change, user->num_days_warning,
		user->num_days_inactive, user->day_expires,
		user->flag));

}

gchar **User_to_row (struct User *user) {

	gchar **row;
	gchar   num[16];
	gint	x;

	struct group *grp;

	row = (char **)malloc(sizeof(char*)*7);
	row[0] = strdup(user->login);
	sprintf(num, "%i", user->uid);
	row[1] = strdup(num);
	row[2] = "\0";
	for (x=0; x<GTK_CLIST(groups_clist)->rows; x++) {
		grp = gtk_clist_get_row_data(GTK_CLIST(groups_clist),x);
		if (grp->gr_gid == user->gid)
			row[2] = strdup(grp->gr_name);
	}
	row[3] = strdup(user->name);
	row[4] = strdup(user->dir);
	row[5] = strdup(user->shell);
	row[6] = NULL;

	return (row);
}

gchar **group_to_row (struct group *grp) {

	gchar **row;
	gchar num[16], user[128];
	long int size, x;

	row = (char **)malloc(sizeof(char*)*5);	
	row[0] = strdup(grp->gr_name);
	row[1] = strdup(grp->gr_passwd);
	sprintf(num, "%i", grp->gr_gid);
	row[2] = strdup(num);
	x=0;
	size = 1;
	while (grp->gr_mem[x]) {
		size += strlen(grp->gr_mem[x]);
		x++;
	}
	row[3] = (char*)malloc(sizeof(char)*(size+x+1));
	strcpy(row[3], "");
	x=0;
	while (grp->gr_mem[x]) {
		sprintf (user, "%s,", grp->gr_mem[x]);
		strcat (row[3], user);
		x++;
	}
	row[4] = NULL;

	return (row);
}
