/*
    gUserMan- A GTK+ User manager
    Copyright (C) 1999 Scott Fritzinger 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* This is all the user's information. it's MUCH easier to work with
   in this format. If you need to extract a passwd* or a spwd* to write 
   to the passwd or shadow-passwd stream, use the
   create_passwd_from_User() / create_spwd_from_User() */

struct User {
	char *login;
	int   uid;
	int   gid;
	char *name; /* actually the gecos field */
	char *dir;
	char *shell;
	char *password;
	long int date_last_change;
	long int min_days_change;
	long int max_days_change;
	long int num_days_warning;
	long int num_days_inactive;
	long int day_expires;
	unsigned long int flag;
};

/* Frees a pwent struct */
	void		free_passwd(struct passwd *pwent);
	void		free_spwd  (struct spwd *spwent);
	void		free_User  (struct User *user);
	void		free_group (struct group *grp);

/* Creates a struct from the information */
	struct passwd   *create_passwd (char *login, char *passwd, 
					int uid, int gid, char *name,
					char *home, char *shell);

	struct spwd 	*create_spwd (char *login, char *passwd,
                        		long int date_last_change,
                        		long int min_days_change,
                        		long int max_days_change,
                        		long int num_days_warning,
                        		long int num_days_inactive,
                        		long int day_expires,
                        		unsigned long int flag);

	struct group	*create_group (char *name, char *password,
					int gid, GSList *members);

	struct User	*create_User (struct passwd *pwent,
				      	struct spwd   *spwent);

/* Copies a passwd struct */
	struct passwd	*copy_passwd (struct passwd *pwent);
	struct spwd	*copy_spwd   (struct spwd *spwent);
	struct group	*copy_group  (struct group *grp);

/* Creates a struct User* from a struct passwd and struct spwd */
	struct passwd 	*create_passwd_from_User (struct User *user);
	struct spwd 	*create_spwd_from_User (struct User *user);

/* Converts a struct passwd* to a row for clist */
	gchar 	       **User_to_row (struct User *user);
	gchar	       **group_to_row (struct group *grp);	
